@@ -15,10 +15,13 @@ from .objective import (
     CsvHistory,
     Hdf5History,
     OptimizerHistory,
-    Objective,
     AmiciObjective,
-    PetabImporter)
+    Objective,
+    NegLogPriors,
+    ObjectiveBase)
 from .problem import Problem
+from .petab import (
+    PetabImporter)
 from . import startpoint
 from .result import (
     Result,
@@ -31,12 +34,24 @@ from .optimize import (
     OptimizerResult,
     Optimizer,
     ScipyOptimizer,
+    IpoptOptimizer,
     DlibOptimizer,
     PyswarmOptimizer)
 from .profile import (
     parameter_profile,
     ProfileOptions,
     ProfilerResult)
+from .sampling import (
+    sample,
+    Sampler,
+    InternalSampler,
+    MetropolisSampler,
+    AdaptiveMetropolisSampler,
+    ParallelTemperingSampler,
+    AdaptiveParallelTemperingSampler,
+    Pymc3Sampler,
+    McmcPtResult,
+    geweke_test)
 from .engine import (
     SingleCoreEngine,
     MultiThreadEngine,
