@@ -1,17 +1,44 @@
 import numpy as np
 import pandas as pd
+import numbers
 import copy
 import time
 import os
 import abc
-from typing import Any, Dict, Iterable, List, Optional, Tuple, Sequence, Union
+from typing import Any, Dict, List, Tuple, Sequence, Union
 
 from .constants import (
     MODE_FUN, MODE_RES, FVAL, GRAD, HESS, RES, SRES, CHI2, SCHI2, TIME,
     N_FVAL, N_GRAD, N_HESS, N_RES, N_SRES, X)
 from .util import res_to_chi2, sres_to_schi2, sres_to_fim
 
-ResultType = Dict[str, Union[float, np.ndarray]]
+ResultDict = Dict[str, Union[float, np.ndarray]]
+MaybeArray = Union[np.ndarray, 'np.nan']
+
+
+def trace_wrap(f):
+    """
+    Wrapper around trace getters that transforms input `ix` vectors to a valid
+    index list, and reduces for integer `ix` the output to a single value.
+    """
+    def wrapped_f(
+            self, ix: Union[Sequence[int], int, None] = None
+    ) -> Union[Sequence[Union[float, MaybeArray]], Union[float, MaybeArray]]:
+        # whether to reduce the output
+        reduce = isinstance(ix, numbers.Integral)
+        # default: full list
+        if ix is None:
+            ix = np.arange(0, len(self), dtype=int)
+        # turn every input into an index list
+        if reduce:
+            ix = np.array([ix], dtype=int)
+        # obtain the trace
+        trace = f(self, ix)
+        # reduce the output
+        if reduce:
+            trace = trace[0]
+        return trace
+    return wrapped_f
 
 
 class HistoryOptions(dict):
@@ -106,7 +133,7 @@ class HistoryOptions(dict):
         return options
 
     def create_history(
-            self, id: str, x_names: Iterable[str]
+            self, id: str, x_names: Sequence[str]
     ) -> 'History':
         """Factory method creating a :class:`History` object.
 
@@ -148,12 +175,16 @@ class HistoryBase(abc.ABC):
     functionality.
     """
 
+    def __len__(self):
+        """Number of stored entries in the history"""
+        raise NotImplementedError()
+
     def update(
             self,
             x: np.ndarray,
             sensi_orders: Tuple[int, ...],
             mode: str,
-            result: ResultType
+            result: ResultDict
     ) -> None:
         """Update history after a function evaluation.
 
@@ -203,40 +234,94 @@ class HistoryBase(abc.ABC):
         """Start time."""
         raise NotImplementedError()
 
-    def get_x_trace(self) -> Sequence[np.ndarray]:
-        """Parameter trace."""
+    def get_x_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[np.ndarray], np.ndarray]:
+        """Parameters.
+
+        Takes as parameter an index or indices and returns corresponding trace
+        values. If only a single value is requested, the list is flattened.
+        """
         raise NotImplementedError()
 
-    def get_fval_trace(self) -> Sequence[float]:
-        """Function value trace."""
+    def get_fval_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[float], float]:
+        """Function values.
+
+        Takes as parameter an index or indices and returns corresponding trace
+        values. If only a single value is requested, the list is flattened.
+        """
         raise NotImplementedError()
 
-    def get_grad_trace(self) -> Sequence[np.ndarray]:
-        """Gradient trace."""
+    def get_grad_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        """Gradients.
+
+        Takes as parameter an index or indices and returns corresponding trace
+        values. If only a single value is requested, the list is flattened.
+        """
         raise NotImplementedError()
 
-    def get_hess_trace(self) -> Sequence[np.ndarray]:
-        """Hessian trace."""
+    def get_hess_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        """Hessians.
+
+        Takes as parameter an index or indices and returns corresponding trace
+        values. If only a single value is requested, the list is flattened.
+        """
         raise NotImplementedError()
 
-    def get_res_trace(self) -> Sequence[np.ndarray]:
-        """Residual trace."""
+    def get_res_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        """Residuals.
+
+        Takes as parameter an index or indices and returns corresponding trace
+        values. If only a single value is requested, the list is flattened.
+        """
         raise NotImplementedError()
 
-    def get_sres_trace(self) -> Sequence[np.ndarray]:
-        """Residual sensitivity trace."""
+    def get_sres_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        """Residual sensitivities.
+
+        Takes as parameter an index or indices and returns corresponding trace
+        values. If only a single value is requested, the list is flattened.
+        """
         raise NotImplementedError()
 
-    def get_chi2_trace(self) -> Sequence[np.ndarray]:
-        """Chi2 value trace."""
+    def get_chi2_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[float], float]:
+        """Chi2 values.
+
+        Takes as parameter an index or indices and returns corresponding trace
+        values. If only a single value is requested, the list is flattened.
+        """
         raise NotImplementedError()
 
-    def get_schi2_trace(self, t: Optional[int] = None) -> Sequence[np.ndarray]:
-        """Chi2 value sensitivity trace."""
+    def get_schi2_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        """Chi2 sensitivities.
+
+        Takes as parameter an index or indices and returns corresponding trace
+        values. If only a single value is requested, the list is flattened.
+        """
         raise NotImplementedError()
 
-    def get_time_trace(self, t: Optional[int] = None) -> Sequence[np.ndarray]:
-        """Execution time trace."""
+    def get_time_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[float], float]:
+        """Cumulative execution times.
+
+        Takes as parameter an index or indices and returns corresponding trace
+        values. If only a single value is requested, the list is flattened.
+        """
         raise NotImplementedError()
 
 
@@ -267,7 +352,7 @@ class History(HistoryBase):
             x: np.ndarray,
             sensi_orders: Tuple[int, ...],
             mode: str,
-            result: ResultType
+            result: ResultDict
     ) -> None:
         """Update history after a function evaluation.
 
@@ -347,51 +432,81 @@ class MemoryHistory(History):
         self._trace_keys = {X, FVAL, GRAD, HESS, RES, SRES, CHI2, SCHI2, TIME}
         self._trace: Dict[str, Any] = {key: [] for key in self._trace_keys}
 
+    def __len__(self):
+        return len(self._trace[TIME])
+
     def update(
             self,
             x: np.ndarray,
             sensi_orders: Tuple[int, ...],
             mode: str,
-            result: ResultType
+            result: ResultDict
     ) -> None:
         super().update(x, sensi_orders, mode, result)
-        self._update_trace(x, sensi_orders, mode, result)
+        self._update_trace(x, mode, result)
 
-    def _update_trace(self, x, sensi_orders, mode, result):
+    def _update_trace(self, x, mode, result):
         """Update internal trace representation."""
-        ret = extract_values(sensi_orders, mode, result, self.options)
+        ret = extract_values(mode, result, self.options)
         for key in self._trace_keys - {X, TIME}:
             self._trace[key].append(ret[key])
         used_time = time.time() - self._start_time
         self._trace[X].append(x)
         self._trace[TIME].append(used_time)
 
-    def get_x_trace(self) -> Sequence[np.ndarray]:
-        return self._trace[X]
+    @trace_wrap
+    def get_x_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[np.ndarray], np.ndarray]:
+        return [self._trace[X][i] for i in ix]
 
-    def get_fval_trace(self) -> Sequence[float]:
-        return self._trace[FVAL]
+    @trace_wrap
+    def get_fval_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[float], float]:
+        return [self._trace[FVAL][i] for i in ix]
 
-    def get_grad_trace(self) -> Sequence[np.ndarray]:
-        return self._trace[GRAD]
+    @trace_wrap
+    def get_grad_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        return [self._trace[GRAD][i] for i in ix]
 
-    def get_hess_trace(self) -> Sequence[np.ndarray]:
-        return self._trace[HESS]
+    @trace_wrap
+    def get_hess_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        return [self._trace[HESS][i] for i in ix]
 
-    def get_res_trace(self) -> Sequence[np.ndarray]:
-        return self._trace[RES]
+    @trace_wrap
+    def get_res_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        return [self._trace[RES][i] for i in ix]
 
-    def get_sres_trace(self) -> Sequence[np.ndarray]:
-        return self._trace[SRES]
+    @trace_wrap
+    def get_sres_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        return [self._trace[SRES][i] for i in ix]
 
-    def get_chi2_trace(self) -> Sequence[np.ndarray]:
-        return self._trace[CHI2]
+    @trace_wrap
+    def get_chi2_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[float], float]:
+        return [self._trace[CHI2][i] for i in ix]
 
-    def get_schi2_trace(self, t: Optional[int] = None) -> Sequence[np.ndarray]:
-        return self._trace[SCHI2]
+    @trace_wrap
+    def get_schi2_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        return [self._trace[SCHI2][i] for i in ix]
 
-    def get_time_trace(self, t: Optional[int] = None) -> Sequence[np.ndarray]:
-        return self._trace[TIME]
+    @trace_wrap
+    def get_time_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[float], float]:
+        return [self._trace[TIME][i] for i in ix]
 
 
 class CsvHistory(History):
@@ -405,12 +520,15 @@ class CsvHistory(History):
         Parameter names.
     options:
         History options.
+    load_from_file:
+        If True, history will be initialized from data in the specified file
     """
 
     def __init__(self,
                  file: str,
-                 x_names: Iterable[str] = None,
-                 options: Union[HistoryOptions, Dict] = None):
+                 x_names: Sequence[str] = None,
+                 options: Union[HistoryOptions, Dict] = None,
+                 load_from_file: bool = False):
         super().__init__(options=options)
         self.x_names = x_names
         self._trace: Union[pd.DataFrame, None] = None
@@ -421,15 +539,41 @@ class CsvHistory(History):
             dirname = os.path.dirname(self.file)
             os.makedirs(dirname, exist_ok=True)
 
+        if load_from_file:
+            trace = pd.read_csv(self.file, header=[0, 1], index_col=0)
+            # replace 'nan' in cols with np.NAN
+            cols = pd.DataFrame(trace.columns.to_list())
+            cols[cols == 'nan'] = np.NaN
+            trace.columns = pd.MultiIndex.from_tuples(
+                cols.to_records(index=False).tolist()
+            )
+            for col in trace.columns:
+                # transform strings to np.ndarrays
+                trace[col] = trace[col].apply(string2ndarray)
+
+            self._trace = trace
+            self.x_names = trace[X].columns
+            self._update_counts_from_trace()
+
+    def __len__(self):
+        return len(self._trace)
+
+    def _update_counts_from_trace(self):
+        self._n_fval = self._trace[('n_fval', np.NaN)].max()
+        self._n_grad = self._trace[('n_grad', np.NaN)].max()
+        self._n_hess = self._trace[('n_hess', np.NaN)].max()
+        self._n_res = self._trace[('n_res', np.NaN)].max()
+        self._n_sres = self._trace[('n_sres', np.NaN)].max()
+
     def update(
             self,
             x: np.ndarray,
             sensi_orders: Tuple[int, ...],
             mode: str,
-            result: ResultType
+            result: ResultDict
     ) -> None:
         super().update(x, sensi_orders, mode, result)
-        self._update_trace(x, sensi_orders, mode, result)
+        self._update_trace(x, mode, result)
 
     def finalize(self):
         """Finalize history. Called after a run."""
@@ -438,9 +582,8 @@ class CsvHistory(History):
 
     def _update_trace(self,
                       x: np.ndarray,
-                      sensi_orders: Tuple[int],
                       mode: str,
-                      result: ResultType):
+                      result: ResultDict):
         """
         Update and possibly store the trace.
         """
@@ -452,7 +595,7 @@ class CsvHistory(History):
             self._init_trace(x)
 
         # extract function values
-        ret = extract_values(sensi_orders, mode, result, self.options)
+        ret = extract_values(mode, result, self.options)
 
         used_time = time.time() - self._start_time
 
@@ -480,9 +623,9 @@ class CsvHistory(History):
 
         for var, val in {X: x, GRAD: ret[GRAD], SCHI2: ret[SCHI2]}.items():
             if var == X or self.options[f'trace_record_{var}']:
-                row[var] = val if val is not None else np.NaN
+                row[var] = val
             else:
-                row[(var, float('nan'))] = None
+                row[(var, float('nan'))] = np.NaN
 
         self._trace = self._trace.append(row)
 
@@ -513,7 +656,6 @@ class CsvHistory(History):
                 columns.extend([(var,)])
 
         # TODO: multi-index for res, sres, hess
-
         self._trace = pd.DataFrame(columns=pd.MultiIndex.from_tuples(columns),
                                    dtype='float64')
 
@@ -530,16 +672,13 @@ class CsvHistory(History):
         }
 
         for var, dtype in trace_dtypes.items():
-            self._trace[(var, float('nan'))] = \
-                self._trace[(var, float('nan'))].astype(dtype)
+            self._trace[(var, np.NaN)] = \
+                self._trace[(var, np.NaN)].astype(dtype)
 
     def _save_trace(self, finalize: bool = False):
         """
         Save to file via pd.DataFrame.to_csv() if `self.storage_file` is
         not None and other conditions apply.
-
-        .. note::
-            Format might be revised when storage is implemented.
         """
         if self.file is None:
             return
@@ -555,9 +694,59 @@ class CsvHistory(History):
                 )
             trace_copy.to_csv(self.file)
 
-    def get_fval_trace(self) -> pd.Series:
-        # TODO implement the other methods
-        return self._trace[FVAL]
+    @trace_wrap
+    def get_x_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[np.ndarray], np.ndarray]:
+        return list(self._trace[X].values[ix])
+
+    @trace_wrap
+    def get_fval_trace(
+            self, ix: Union[int, Sequence[int], None]
+    ) -> Union[Sequence[float], float]:
+        return list(self._trace[(FVAL, np.nan)].values[ix])
+
+    @trace_wrap
+    def get_grad_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        return list(self._trace[GRAD].values[ix])
+
+    @trace_wrap
+    def get_hess_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        return list(self._trace[(HESS, np.nan)].values[ix])
+
+    @trace_wrap
+    def get_res_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        return list(self._trace[(RES, np.nan)].values[ix])
+
+    @trace_wrap
+    def get_sres_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        return list(self._trace[(SRES, np.nan)].values[ix])
+
+    @trace_wrap
+    def get_chi2_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[float], float]:
+        return list(self._trace[(CHI2, np.nan)].values[ix])
+
+    @trace_wrap
+    def get_schi2_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[MaybeArray], MaybeArray]:
+        return list(self._trace[SCHI2].values[ix])
+
+    @trace_wrap
+    def get_time_trace(
+            self, ix: Union[int, Sequence[int], None] = None
+    ) -> Union[Sequence[float], float]:
+        return list(self._trace[(TIME, np.nan)].values[ix])
 
 
 class Hdf5History(History):
@@ -586,7 +775,7 @@ class Hdf5History(History):
             x: np.ndarray,
             sensi_orders: Tuple[int, ...],
             mode: str,
-            result: ResultType
+            result: ResultDict
     ) -> None:
         # TODO implement
         raise NotImplementedError()
@@ -595,58 +784,45 @@ class Hdf5History(History):
         # TODO implement
         raise NotImplementedError()
 
-    def get_x_trace(self) -> Sequence[np.ndarray]:
-        # TODO implement
-        raise NotImplementedError()
 
-    def get_fval_trace(self) -> Sequence[float]:
-        # TODO implement
-        raise NotImplementedError()
-
-    def get_grad_trace(self) -> Sequence[np.ndarray]:
-        # TODO implement
-        raise NotImplementedError()
-
-    def get_hess_trace(self) -> Sequence[np.ndarray]:
-        # TODO implement
-        raise NotImplementedError()
-
-    def get_res_trace(self) -> Sequence[np.ndarray]:
-        # TODO implement
-        raise NotImplementedError()
-
-    def get_sres_trace(self) -> Sequence[np.ndarray]:
-        # TODO implement
-        raise NotImplementedError()
-
-    def get_chi2_trace(self) -> Sequence[np.ndarray]:
-        # TODO implement
-        raise NotImplementedError()
-
-    def get_schi2_trace(self, t: Optional[int] = None) -> Sequence[np.ndarray]:
-        # TODO implement
-        raise NotImplementedError()
-
-    def get_time_trace(self, t: Optional[int] = None) -> Sequence[np.ndarray]:
-        # TODO implement
-        raise NotImplementedError()
-
-
-class OptimizerHistory(HistoryBase):
+class OptimizerHistory:
     """
-    Objective call history. Also handles saving of intermediate results.
+    Objective call history. Container around a History object, which keeps
+    track of optimal values.
 
     Attributes
     ----------
     fval0, fval_min:
         Initial and best function value found.
+    chi20, chi2_min:
+        Initial and best chi2 value found.
     x0, x_min:
         Initial and best parameters found.
+    grad_min:
+        gradient for best parameters
+    hess_min:
+        hessian (approximation) for best parameters
+    res_min:
+        residuals for best parameters
+    sres_min:
+        residual sensitivities for best parameters
+
+    Parameters
+    ----------
+    history:
+        History object to attach to this container. This history object
+        implements the storage of the actual history.
+    x0:
+        Initial values for optimization
+    generate_from_history:
+        If set to true, this function will try to fill attributes of this
+        function based on the provided history
     """
 
     def __init__(self,
                  history: History,
-                 x0: np.ndarray) -> None:
+                 x0: np.ndarray,
+                 generate_from_history: bool = False) -> None:
         self.history: History = history
 
         # initial point
@@ -661,56 +837,105 @@ class OptimizerHistory(HistoryBase):
         self.res_min: Union[np.ndarray, None] = None
         self.sres_min: Union[np.ndarray, None] = None
 
+        if generate_from_history:
+            self._compute_vals_from_trace()
+
     def update(self,
                x: np.ndarray,
                sensi_orders: Tuple[int],
                mode: str,
-               result: ResultType) -> None:
+               result: ResultDict) -> None:
         """Update history and best found value."""
         self.history.update(x, sensi_orders, mode, result)
-        self._update_vals(x, sensi_orders, mode, result)
+        self._update_vals(x, result)
 
     def finalize(self):
         self.history.finalize()
 
     def _update_vals(self,
                      x: np.ndarray,
-                     sensi_orders: Tuple[int],
-                     mode: str,
-                     result: ResultType):
+                     result: ResultDict):
         """
         Update initial and best function values.
         """
         # update initial point
-        if self.fval0 is None and np.allclose(x, self.x0) \
-                and 0 in sensi_orders:
-            if mode == MODE_FUN:
-                self.fval0 = result[FVAL]
-                self.x0 = x
-            else:  # mode == MODE_RES:
-                chi2 = res_to_chi2(result[RES])
-                self.fval0 = result.get(FVAL, chi2)
-                self.x0 = x
+        if np.allclose(x, self.x0):
+            if self.fval0 is None:
+                self.fval0 = result.get(FVAL, None)
+            self.x0 = x
 
         # update best point
-        if 0 in sensi_orders:
-            # extract function value
-            if mode == MODE_FUN:
-                fval = result[FVAL]
-            else:  # mode == MODE_RES:
-                chi2 = res_to_chi2(result[RES])
-                fval = result.get(FVAL, chi2)
-            # store value
-            if fval < self.fval_min:
-                self.fval_min = fval
-                self.x_min = x
-                self.grad_min = result.get(GRAD)
-                self.hess_min = result.get(HESS)
-                self.res_min = result.get(RES)
-                self.sres_min = result.get(SRES)
+        fval = result.get(FVAL, None)
+        grad = result.get(GRAD, None)
+        hess = result.get(HESS, None)
+        res = result.get(RES, None)
+        sres = result.get(SRES, None)
+
+        if fval is not None and fval < self.fval_min:
+            self.fval_min = fval
+            self.x_min = x
+            self.grad_min = grad
+            self.hess_min = hess
+            self.res_min = res
+            self.sres_min = sres
+
+        # sometimes sensitivities are evaluated on subsequent calls. We can
+        # identify this situation by checking that x hasn't changed
+        if np.all(self.x_min == x):
+            if self.grad_min is None and grad is not None:
+                self.grad_min = grad
+            if self.hess_min is None and hess is not None:
+                self.hess_min = hess
+            if self.res_min is None and res is not None:
+                self.res_min = res
+            if self.sres_min is None and sres is not None:
+                self.sres_min = sres
+
+    def _compute_vals_from_trace(self):
+        # some optimizers may evaluate hess+grad first to compute trust region
+        # etc
+        max_init_iter = 3
+        for it in range(min(len(self.history), max_init_iter)):
+            candidate = self.history.get_fval_trace(it)
+            if not np.isnan(candidate) \
+                    and np.allclose(self.history.get_x_trace(it), self.x0):
+                self.fval0 = candidate
+                break
+
+        # we prioritize fval over chi2 as fval is written whenever possible
+        ix_min = np.nanargmin(self.history.get_fval_trace())
+        # np.argmin returns ndarray when multiple minimal values are found, we
+        # generally want the first occurence
+        if isinstance(ix_min, np.ndarray):
+            ix_min = ix_min[0]
+
+        for var in ['fval', 'chi2', 'x']:
+            self.extract_from_history(var, ix_min)
+
+        if self.history.options.trace_record_res:
+            self.extract_from_history('res', ix_min)
+
+        for var in ['grad', 'sres', 'hess']:
+            target = f'{var}_min'  # attribute in self we want to set
+            ix_try = ix_min + 1  # index we try after ix_min doesnt work
+            if getattr(self.history.options, f'trace_record_{var}'):
+                self.extract_from_history(var, ix_min)
+                if getattr(self, target) is None \
+                        and ix_try < len(self.history) \
+                        and np.allclose(self.history.get_x_trace(ix_min),
+                                        self.history.get_x_trace(ix_try)):
+                    # gradient/sres typically evaluated on the next call
+                    # so we check if x remains the same and if yes try to
+                    # extract from the next
+                    self.extract_from_history(var, ix_try)
+
+    def extract_from_history(self, var, ix):
+        val = getattr(self.history, f'get_{var}_trace')(ix)
+        if not np.all(np.isnan(val)):
+            setattr(self, f'{var}_min', val)
 
 
-def ndarray2string_full(x: np.ndarray):
+def ndarray2string_full(x: Union[np.ndarray, None]) -> Union[str, None]:
     """
     Helper function that converts numpy arrays to string with 16 digit
     numerical precision and no truncation for large arrays
@@ -725,53 +950,71 @@ def ndarray2string_full(x: np.ndarray):
     x:
         array as string
     """
-    if x is None:
-        return None
-    return np.array2string(x, threshold=len(x), precision=16,
+    if not isinstance(x, np.ndarray):
+        return x
+    return np.array2string(x, threshold=x.size, precision=16,
                            max_line_width=np.inf)
 
 
-def extract_values(sensi_orders: Tuple[int, ...],
-                   mode: str,
-                   result: ResultType,
+def string2ndarray(x: Union[str, float]) -> Union[np.ndarray, float]:
+    """
+    Helper function that converts string to numpy arrays
+
+    Parameters
+    ----------
+    x:
+        array to convert
+
+    Returns
+    -------
+    x:
+        array as np.ndarray
+    """
+    if not isinstance(x, str):
+        return x
+    if x.startswith('[['):
+        return np.vstack([
+            np.fromstring(xx, sep=' ')
+            for xx in x[2:-2].split(']\n [')
+        ])
+    else:
+        return np.fromstring(x[1:-1], sep=' ')
+
+
+def extract_values(mode: str,
+                   result: ResultDict,
                    options: HistoryOptions) -> Dict:
     """Extract values to record from result."""
-    if mode == MODE_FUN:
-        fval = np.NaN if 0 not in sensi_orders \
-            else result.get(FVAL, np.NaN)
-        grad = None if not options.trace_record_grad \
-            or 1 not in sensi_orders \
-            else result.get(GRAD, None)
-        hess = None if not options.trace_record_hess \
-            or 2 not in sensi_orders \
-            else result.get(HESS, None)
-        res = None
-        sres = None
-        chi2 = np.NaN
-        schi2 = None
-    else:  # mode == MODE_RES
+    ret = {}
+    ret_vars = [FVAL, GRAD, HESS, RES, SRES, CHI2, SCHI2]
+    for var in ret_vars:
+        if options.get(f'trace_record_{var}', True) and var in result:
+            ret[var] = result[var]
+
+    # write values that weren't set yet with alternative methods
+    if mode == MODE_RES:
         res_result = result.get(RES, None)
         sres_result = result.get(SRES, None)
-        chi2 = np.NaN if not options.trace_record_chi2 \
-            or 0 not in sensi_orders \
-            else res_to_chi2(res_result)
-        schi2 = None if not options.trace_record_schi2 \
-            or 1 not in sensi_orders \
-            else sres_to_schi2(res_result, sres_result)
-        fval = np.NaN if 0 not in sensi_orders \
-            else result.get(FVAL, chi2)
-        grad = None if not options.trace_record_grad \
-            or 1 not in sensi_orders \
-            else schi2
-        hess = None if not options.trace_record_hess \
-            or 1 not in sensi_orders \
-            else sres_to_fim(sres_result)
-        res = None if not options.trace_record_res \
-            or 0 not in sensi_orders \
-            else res_result
-        sres = None if not options.trace_record_sres \
-            or 1 not in sensi_orders \
-            else sres_result
+        chi2 = res_to_chi2(res_result)
+        schi2 = sres_to_schi2(res_result, sres_result)
+        fim = sres_to_fim(sres_result)
+        alt_values = {CHI2: chi2, SCHI2: schi2, HESS: fim}
+        if schi2 is not None:
+            alt_values[GRAD] = 0.5 * schi2
 
-    return {FVAL: fval, GRAD: grad, HESS: hess, RES: res, SRES: sres,
-            CHI2: chi2, SCHI2: schi2}
+        # filter according to options
+        alt_values = {
+            key: val
+            for key, val in alt_values.items()
+            if options.get(f'trace_record_{key}', True)
+        }
+        for var, val in alt_values.items():
+            if val is not None:
+                ret[var] = ret.get(var, val)
+
+    # set everything missing to NaN
+    for var in ret_vars:
+        if var not in ret:
+            ret[var] = np.NaN
+
+    return ret
